#include "822lib.h"

/* tokenize a rfc 822 field.
 * takes "in", creates "out".
 * out format:
 * \0token1C\0token2C[...]\0tokenxC
 * where C is one of:
 * C: comment `(some (other))' without the outer ( and )
 * D: domain literal `[....]' including [ and ]
 * Q: quoted string  `"..."' without "
 * S: single character (specials + sp + tab)
 * W: word
 * written after a deep look inside djb's mess822_token.c
 */
#define BUFF_ADD(c) do { \
	*bufp++=c; if (bufp==bufe) { \
			if (!uostr_add_mem(out,buf,bufe-buf)) return R822_ERR_NOMEM; \
			bufp=buf; \
	} \
} while(0)
int
r822_lex(uostr_t *out, uostr_t *in)
{
	char buf[128];
	char *bufe=buf+sizeof(buf);
	char *bufp=buf;
	char *p, *e;
	e=in->data+in->len;
	p=in->data;
	uostr_fcut(out);
	while (p!=e) {
		char c;
		c=*p++;
		switch (c) {
			int level; /* comments */
		case '\n': break;
		case '(': 
			BUFF_ADD('\0');
			level=1;
			while (p!=e) {
				c = *p++;
				if (c==')') {
					level--; if (!level) break;
					BUFF_ADD(c);
					continue;
				}
				if (c=='\\' && p!=e) c=*p++;
				else if (c=='(') 
					level++;
				BUFF_ADD(c);
			}
			BUFF_ADD('C');
			break;
		case '[':
			BUFF_ADD('\0');
			BUFF_ADD('[');
			while (p!=e) {
				c = *p++;
				if (c==']') break;
				if (c=='\\' && p!=e) c=*p++;
				BUFF_ADD(c);
			}
			BUFF_ADD(']');
			BUFF_ADD('D');
			break;
		case '"':
			BUFF_ADD('\0');
			while (p!=e) {
				c = *p++;
				if (c=='"') break;
				if (c=='\\' && p!=e) c=*p++;
				BUFF_ADD(c);
			}
			BUFF_ADD('Q');
			break;
		case ')': case '<': case '>': case '@': 
		case ',': case ';': case ':': case '.':
		case ']':
			BUFF_ADD('\0');
			BUFF_ADD(c);
			BUFF_ADD('S');
			break;
		case ' ':
		case '\t':
			BUFF_ADD('\0');
			BUFF_ADD(' ');
			BUFF_ADD('S');
			break;
		default:
			BUFF_ADD('\0');
			while (1) {
				if (c=='\\' && p!=e) c=*p++;
				if (c!='\n')
					BUFF_ADD(c);
				if (p==e) break;
				c = *p;
				if (c=='(' || c==')' || c=='<' || c=='>' 
					|| c=='@' || c==',' || c==';' || c==':' 
					|| c=='.' || c==']' || c==' ' || c=='\t'
					|| c=='"')
					break;
				p++;
			}
			BUFF_ADD('W');
			break;
		} /* switch */
	}
	if (bufp!=buf)
		if (!uostr_add_mem(out,buf,bufp-buf)) return R822_ERR_NOMEM;
	return R822_OK;
}
