/*
    Copyright (C) 1999 Uwe Ohse

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Contact: uwe@ohse.de, Uwe Ohse @ DU3 (mausnet)
*/
/* Umlaute in Langform wandeln */

#include "uostr.h"
#include "uo_entumlaut.h"

static unsigned char tbl[512];
static int init_done;
#define MKUCHAR(x) ((unsigned char) x)

static void 
init_uwan (void)
{
	unsigned int i;

	for (i = 0; i < 256; i++)
	{
		tbl[i*2] = (unsigned char) i;
		tbl[i*2+1] = (unsigned char) 0;
	}

	tbl[MKUCHAR ('') * 2] = MKUCHAR ('u');
	tbl[MKUCHAR ('') * 2 + 1] = MKUCHAR ('e');
	tbl[MKUCHAR ('') * 2] = MKUCHAR ('U');
	tbl[MKUCHAR ('') * 2 + 1] = MKUCHAR ('e');
	tbl[MKUCHAR ('') * 2] = MKUCHAR ('a');
	tbl[MKUCHAR ('') * 2 + 1] = MKUCHAR ('e');
	tbl[MKUCHAR ('') * 2] = MKUCHAR ('A');
	tbl[MKUCHAR ('') * 2 + 1] = MKUCHAR ('e');
	tbl[MKUCHAR ('') * 2] = MKUCHAR ('o');
	tbl[MKUCHAR ('') * 2 + 1] = MKUCHAR ('e');
	tbl[MKUCHAR ('') * 2] = MKUCHAR ('O');
	tbl[MKUCHAR ('') * 2 + 1] = MKUCHAR ('e');
	tbl[MKUCHAR ('') * 2] = MKUCHAR ('s');
	tbl[MKUCHAR ('') * 2 + 1] = MKUCHAR ('s');

	init_done = 1;
}

uostr_t *
uostr_entumlauten_mem(uostr_t *target,const char *from, size_t len)
{
	const unsigned char *f = (const unsigned char *) from;
	int i;
	unsigned char c;
	unsigned char buf[128]; /* 16 * x */
	unsigned char *to;

	uostr_need(target,len); /* advise */
	uostr_cut(target,0);

	if (!init_done) init_uwan ();

	if (!len) return target;

#define WORK { \
		i=(*f++); \
		if (i<128) *to++=i; \
		else { i *=2 ; *to++ = tbl[i++]; c=tbl[i++]; if (c) *to++=c; } \
	}

	to=buf;
   /*
    * Duffs device - loop unrolling.
    */
	switch (len % 16)
	{
		while (1)
		{
	default:
	case 0:
			WORK
	case 15:
			WORK
	case 14:
			WORK
	case 13:
			WORK
	case 12:
			WORK
	case 11:
			WORK
	case 10:
			WORK
	case 9:
			WORK
	case 8:
			WORK
	case 7:
			WORK
	case 6:
			WORK
	case 5:
			WORK
	case 4:
			WORK
	case 3:
			WORK
	case 2:
			WORK
	case 1:
			WORK
			if (to+16>=buf+sizeof(buf) || ((const char *)f)==from+len) {
				if (!uostr_add_mem(target,(char *)buf,to-buf)) return 0;
				to=buf;
				return target;
			}
		}
	}
#undef WORK
}
